#!/bin/sh

DRIVER=rr640l

echo "Driver Installation"

MACHINE=`uname -m`
KERNEL=`uname -r`
CWD=`dirname $0`
REMOVE_MOD=NO_CONFLICT
KMOD=ko

# From Fedora 12 change the name to initramfs
# so we should test the release.
# It may be best to check dracut to determine initramfs

[ -x /bin/dracut ] && INITRD=initramfs

( echo $KERNEL | grep "^2.4" -s -q ) && { KMOD=o; }

export PATH=/bin:/sbin:/usr/bin:/usr/sbin

if  [ ${DRIVER} = "hptiop" ]; then
	REMOVE_MOD=hptiop
elif [ ${DRIVER} = "hptmv" ] || [ ${DRIVER} = "hptmv6" ] || [ ${DRIVER} = "rr2310_00" ] || [ ${DRIVER} = "rr174x" ] || [ ${DRIVER} = "rr172x" ]; then
    	REMOVE_MOD=sata_mv
elif [ ${DRIVER} = "rr272x_1x" ] || [ ${DRIVER} = "rr274x_3x" ] || [ ${DRIVER} = "rr276x" ] || [ ${DRIVER} = "rr278x" ]; then
    	REMOVE_MOD=mvsas
elif [ ${DRIVER} = "hpt374" ]; then	
	if ( grep "pata_hpt3x2n" /lib/modules/${KERNEL}/modules.dep -s -q ); then
		REMOVE_MOD=pata_hpt3x2n
	elif ( grep "pata_hpt37x" /lib/modules/${KERNEL}/modules.dep -s -q); then
		REMOVE_MOD=pata_hpt37x
	elif ( grep "/hpt366" /lib/modules/${KERNEL}/modules.dep -s -q ); then
		REMOVE_MOD=hpt366
	fi
fi

mkdir /tmp/modules
(cd /tmp/modules; gzip -dc $CWD/modules.cgz | cpio -idum) 1>/dev/null 2>&1 || { echo "Fail to extract drivers!"; exit 1; }

for ver in $(ls /lib/modules/); do
	kver=`basename $ver`
	[ -d /lib/modules/$kver/kernel/fs ] || continue
	mod=`find /tmp/modules/${kver} -type f -name *.${KMOD} 2> /dev/null | head -1`
	vermagic=`modinfo -F vermagic $mod 2> /dev/null`
	[ "$vermagic" = "" ] && vermagic=`strings $mod | grep vermagic`
	( echo "$vermagic" | grep " 386 " -s -q ) && MACHINE=i386
	( echo "$vermagic" | grep " 586 " -s -q ) && MACHINE=i586
	( echo "$vermagic" | grep " 686 " -s -q ) && MACHINE=i686
	( echo "$vermagic" | grep " x86_64 " -s -q ) && MACHINE=x86_64

	if [ -d /lib/modules/${kver} -a -d /tmp/modules/${kver}/${MACHINE}/ ]; then
		echo "Updating ${kver}..."
                REMOVE_MOD_FILE=`grep ${REMOVE_MOD} /lib/modules/${kver}/modules.dep 2>/dev/null |cut -d\: -f1`
		if [ "${REMOVE_MOD_FILE}" != "" ];then
			rm -f ${REMOVE_MOD_FILE} /lib/modules/${kver}/${REMOVE_MOD_FILE}
		fi
		mv /boot/${INITRD}-${kver}.img /boot/${INITRD}-${kver}.img.save 
		cp /tmp/modules/${kver}/${MACHINE}/${DRIVER}.${KMOD} /lib/modules/${kver}/kernel/drivers/scsi/ &&
		/sbin/depmod -a ${kver}

		if [ $? = 1 ]; then
			echo "Postinstall 1 failed! Please reinstall your GNU/Linux system."
			exit
		fi

		( grep -s -q scsi_mod /lib/modules/${kver}/modules.dep ) && PRESCSI="--preload scsi_mod"
		( grep -s -q sd_mod /lib/modules/${kver}/modules.dep ) && PRESD="--preload sd_mod"
		( grep -s -q scsi_wait_scan /lib/modules/${kver}/modules.dep ) && WITHWAIT="--with=scsi_wait_scan"

		# Use dracut(From Fedora 12) to generate initrd if possible.
		if [ -x /bin/dracut ]; then
			/bin/dracut --add-drivers ${DRIVER} -f /boot/${INITRD}-${kver}.img ${kver} 2> /dev/null

		elif [ -x /sbin/mkinitrd ]; then
			if [ ${DRIVER} = ${REMOVE_MOD} ]; then
				/sbin/mkinitrd ${PRESCSI} ${PRESD} --with=${DRIVER} ${WITHWAIT} /boot/${INITRD}-${kver}.img ${kver}
			else
				/sbin/mkinitrd ${PRESCSI} ${PRESD} --with=${DRIVER} ${WITHWAIT} --builtin=${REMOVE_MOD} /boot/${INITRD}-${kver}.img ${kver}
			fi
		else
			echo "Postinstall 2 failed! No tool to generate initrd image."
			exit
		fi

		if [ $? = 1 ]; then
			echo "Postinstall 2 failed! Please reinstall your GNU/Linux system."
			exit
		fi
	fi
done

rm -fr /tmp/modules

echo "Driver installation step 2 completed."
